# midnight_commander

#### Table of Contents

1. [Overview](#overview)
2. [Module Description - What the module does and why it is useful](#module-description)
3. [Setup - The basics of getting started with midnight_commander](#setup)
	* [What midnight_commander affects](#what-midnight_commander-affects)
	* [Setup requirements](#setup-requirements)
	* [Beginning with midnight_commander](#beginning-with-midnight_commander)
4. [Usage - Configuration options and additional functionality](#usage)
5. [Reference - An under-the-hood peek at what the module is doing and how](#reference)
6. [Limitations - OS compatibility, etc.](#limitations)
7. [Development - Guide for contributing to the module](#development)


## Overview

This module installs and globally configures the GNU midnight commander.


## Module Description

This module installs GNU midnight commander (`mc`) –
a free text-user-interface file manager.


## Setup

### What midnight_commander affects

* installs `mc` package
* Optionally: overwrite `/usr/share/mc/mc.ini`
* Optionally: set symlink target for `/etc/mc/skins/default.ini`
  and create `/etc/mc/skins/` for that

### Setup Requirements

This module requires the future parser.
[PP 3.8](https://docs.puppet.com/puppet/3.8/reference/experiments_future.html)).

### Beginning with midnight_commander

Just installing `mc` and doing nothing else:

	class my_custom_software_bundle_wrapper_class {
		class { 'midnight_commander':
			distribute_global_mc_ini     => false,
		}
	}


## Usage

This is how I set up my mc:

	class mc {
		class { 'midnight_commander':
			mc_verbose                   => true,
			mc_use_internal_edit         => false,
			mc_use_internal_view         => false,
			
			mc_confirm_exit              => false,
			mc_skin                      => 'gotar',
			
			mc_kilobyte_si               => false,
			mc_show_backups              => true,
			mc_show_dot_files            => true,
			
			mc_layout_menubar_visible    => false,
			mc_layout_keybar_visible     => false,
			mc_layout_message_visible    => false,
			mc_layout_xterm_title        => false,
			mc_layout_show_mini_info     => false,
		}
	}

Both examples are included in `examples/`.
The default `mc.ini` template gets only populated
with configuration values _you_ specified,
since all of them are `undef` by default.

## Reference

### Classes

* `midnight_commander`: holds the class parameter types
  and [`contain`](https://docs.puppet.com/puppet/latest/reference/function.html#contain)s
  the other classes
* `midnight_commander::install`: installs the `mc` package
* `midnight_commander::config`: distributes the configuration files

### Parameters

The following incorporates help from `/usr/share/mc/help/mc.hlp`.
If I'm not wrong,
they are licensed under [GPL-3+](https://spdx.org/licenses/GPL-3.0+.html).

#### `mc_package_name`
If you need to overwrite the name of the package.
The `package` resource is named `midnight_commander`.
It is only managed,
if the package name is of positive length.

#### `latest_instead_of_present`
`Package['midnight_commander']` is `present` by default.
Set this parameter to true, to ensure it's `latest`.

#### `distribute_global_mc_ini`
Whether to manage the file located at `mc_ini_path`.

#### `mc_ini_path`
The path to the system-wide `mc.ini`.
Unfortunately I did not find any (documented) place
for a system-wide configuration file in `/etc/`
(I did grep for `etc` in `mc(1)`).
So we overwrite the file in `/usr/share/mc/`
though that's not quite where configuration goes.

#### `mc_ini_mode`
Optionally enforce a mode of the `mc.ini` file.
By default `0644` is enforced.

#### `mc_ini_owner`
Optionally the owner of the `mc.ini`.
Normally `root` will be the owner.

#### `mc_ini_group`
Optionally enforce the group of `mc.ini`.
This will be set to `wheel` on `FreeBSD`
[osfamily](https://docs.puppet.com/facter/latest/core_facts.html#osfamily),
`root` otherwise.

#### `mc_ini_template`
The global `mc.ini` is generated by `template()`.
Change it to use another template.
The default template fills only with configuration values,
which _are_ present.
As you start `mc(1)`
all non-specified values are supplemented.
`mc(1)` will then write a _complete_ `mc.ini`
in the user's `~/.config/mc/ini`.

#### `etc_mc_skins_default_symlink`
Holds the target `/etc/mc/skins/default.ini` shall point to.
It is ensured the `skins/` subdirectory exists in `/etc/mc/`.
Normally this parameter is `undef`ined.
If you wanna get rid of the link itself,
assign a zero-length value.

#### `etc_mc_skins_path`
The path to the directory `/etc/mc/skins/`.
(On `FreeBSD` it defaults to `/usr/local/etc/mc/skins/`)
It will will be created before the `default.ini` symlink.

#### `etc_mc_skins_owner`
Optionally the owner of `/etc/mc/skins/`.
That's `root` by default.

#### `etc_mc_skins_group`
Optionally enforce `/etc/mc/skins/` belongs a specific group.
On `FreeBSD` it'll be `wheel`, `root` otherwise.

#### `etc_mc_skins_default_path`
The path of `/etc/mc/skins/default.ini`.
It is set `etc_mc_skins_path` appended with `default.ini`.
The file mode is always set to `0777`.

#### `etc_mc_skins_default_owner`
Same as `etc_mc_skins_owner`.

#### `etc_mc_skins_default_group`
Same as `etc_mc_skins_group`.
	
#### `mc_verbose`
This toggles whether the file copy, rename and delete operations are verbose
(for example, display a dialog box for each operation).
If you have a slow terminal, you may wish to disable the verbose operation.
It is automatically turned off
if the speed of your terminal is less than 9600 bit s⁻¹.

#### `mc_file_op_compute_totals`
If this option is enabled,
`mc(1)` computes
total byte sizes and total number of files
prior to any copy, rename and delete operations.
This will provide you
with a more accurate progress bar at the expense of some speed.
This option has no effect, if verbose operation is disabled.

#### `mc_classic_progressbar`
If this option is enabled,
the progressbar of copy/move/delete operations
is always grown form left to right.
If disabled,
the growing direction of progressbar
follows to direction of copy/move/delete operation:
from left panel to right one and vice versa.
Enabled by default.

#### `mc_auto_fill_mkdir_name`
When you press `F7` to create a new directory,
the input line in popup dialog
will be filled by name of current file or directory in active panel.
Disabled by default.

#### `mc_preallocate_space`
Preallocate space for whole target file, if possible,
before copy operation.
Disabled by default.
	
#### `mc_old_esc_mode`
By default the Midnight Commander treats the `ESC` key as a key prefix.
Therefore, you should press Esc code twice to exit a dialog.
But there is possibility to use a single press of `ESC` key for that action.

By default this option is disabled.
If you'll enable it,
the `ESC` key will act as a prefix key for set up time interval
(see timeout option below),
and if no extra keys have arrived,
then the `ESC` key is interpreted as a cancel key (`ESC ESC`).

#### `mc_old_esc_mode_timeout`
This options is used 
to setup the time interval (in microseconds) for single press of `ESC` key.
By default, this interval is one second (1,000,000 μ″).
Also the timeout can be set
via `KEYBOARD_KEY_TIMEOUT_US` environment variable (also in microseconds),
which has higher priority than the timeout option value.

#### `mc_pause_after_run`
After executing your commands, the Midnight Commander can pause,
so that you can examine the output of the command.
There are three possible settings for this variable:

##### `'never'`
Means that you do not want to see the output of your command.
If you are using the Linux or FreeBSD console or an `xterm(1)`,
you will be able to see the output of the command by typing `C-o`.

##### `'on_dumb_terminals'`
You will get the pause message on terminals
that are not capable of showing the output of the last command executed
(any terminal that is not an xterm or the Linux console).

##### `'always'`
The program will pause after executing all of your commands.

#### `mc_use_internal_edit`
If this option is enabled,
the built-in file editor `mcedit(1)` is used to edit files.
If the option is disabled,
the editor specified in the `EDITOR` environment variable is used.
If no editor is specified, `vi(1)` is used.
See the section on the internal file editor.

#### `mc_use_internal_view`
If this option is enabled,
the built-in file viewer `mcview(1)` is used to edit files.
If the option is disabled,
the pager specified in the `PAGER` environment variable is used.
If no pager is specified, the `view` command is used.

#### `mc_editor_ask_filename_b_edit`
Ask new file name before editing with `mcedit(1)`.

#### `mc_auto_menu`
If this option is enabled,
the user menu will be invoked at startup.
Useful for building menus for non-unixers.

#### `mc_drop_menus`
When this option is enabled,
the pull down menus will be activated as soon as you press the `F9` key.
Otherwise, you will only get the menu title,
and you will have to activate the menu
either with the arrow keys or with the hotkeys.
It is recommended if you are using hotkeys.

#### `mc_shell_patterns`
By default the select, unselect and filter commands
will use shell-like regular expressions.
The following conversions are performed to achieve this:
`*` is replaced by `.*` (zero or more characters);
the `?` is replaced by `.` (exactly one character)
and `.` by the literal dot.
If the option is disabled,
then the regular expressions are the ones described in `ed(1)`.

#### `mc_show_all_if_ambiguous`
By default the Midnight Commander pops up all possible completions
if the completion is ambiguous
only when you press `Alt-Tab` for the second time.
For the first time,
it just completes as much as possible and beeps in the case of ambiguity.
Enable this option
if you want to see all possible completions
even after pressing `Alt-Tab` the first time.

#### `mc_nice_rotating_dash`
If this option is enabled,
the midnight commander shows a rotating dash in the upper right corner
as a work in progress indicator.

#### `mc_cd_symlinks`
This option, if set, causes the Midnight Commander
to follow the logical chain of directories
when changing current directory
either in the panels, or using the `cd` command.
This is the default behavior of `bash(1)`.
When unset, the Midnight Commander follows the real directory structure,
so `cd ..` provided you've entered that directory through a link
will move you to the current directory's real parent
and not to the directory where the link was present.

#### `mc_safe_delete`
If this option is enabled,
deleting files and directory hotlist entries unintentionally
becomes more difficult.
The default selection in the confirmation dialogs for deletion
changes from _Yes_ to _No_.
This option is disabled by default.

#### `mc_auto_save_setup`
If this option is enabled,
when you exit the Midnight Commander
the configurable options of the Midnight Commander
are saved in the `~/.config/mc/ini` file.

#### `mc_confirm_delete`
Whether deletions have to be confirmed.

#### `mc_confirm_overwrite`
Confirm overwrite.

#### `mc_confirm_execute`
Whether to confirm executions of selected files.

#### `mc_confirm_exit`
Whether it is necessary to confirm quitting `mc(1)`.

#### `mc_confirm_dir_hotlist_del`
Whether directory hotlist deletion has to be confirmed.

#### `mc_confirm_history_cleanup`
Confirm history cleanup.

#### `mc_skin`
The skin name. See also `mc(1)`.

#### `mc_use_8th_bit_as_meta`
Full 8bits input.

#### `mc_vfs_timeout`
Timeout for freeing VFSs (sec).

#### `mc_ftpfs_directory_timeout`
FTP directory cache timeout (sec)

#### `mc_ftpfs_always_use_proxy`
Whether to always use an FTP proxy.

#### `mc_use_netrc`
Use `~/.netrc` (see `netrc(5)`).

#### `mc_ftpfs_use_pasconex`
Enables using FTP passive mode,
when the connection for data transfer is initiated by the client,
not by the server.
This option is recommended and enabled by default.
If this option is turned off,
the data connection is initiated by the server.
This may not work with some firewalls.

#### `mc_ftpfs_use_pasconex_x_proxy`
Use passive mode over proxy.

#### `mc_editor_option_auto_par_fmt`
Wrap mode: Dynamic paragraphing.

#### `mc_editor_option_typewr_wrap`
Wrap mode: type writer wrap.

#### `mc_editor_fake_half_tabs`
Fake half tabs.

#### `mc_editor_backspc_throu_tabs`
Backspace through tabs.

#### `mc_editor_fill_tabs_w_spaces`
Fill tabs with spaces.

#### `mc_editor_tab_spacing`
Tab spacing width.

#### `mc_editor_return_d_auto_indnt`
Return does autoindent.

#### `mc_editor_edit_confirm_save`
Confirm before saving.

#### `mc_editor_option_sav_position`
Save file position.

#### `mc_editor_visible_spaces`
Visible trailing spaces.

#### `mc_editor_visible_tabs`
Visible tabs.

#### `mc_editor_syntax_highlighting`
Enable syntax highlighting.

#### `mc_editor_cursr_after_ins_blk`
Place the cursor after inserted block.

#### `mc_editor_persistnt_selexions`
Persistent selections.

#### `mc_editor_cursor_beyond_eol`
Cursor can go beyond end of line.

#### `mc_editor_group_undo`
Group undo.

#### `mc_editor_word_wrap_ln_length`
Word wrap line length.

#### `mc_editor_option_sav_mode`
Valid save modes are `'quick_save'`, `'safe_save'` and
`'do_backups_with_extension'`.

#### `mc_editor_backup_extension`
Only has an effect if `mc_editor_option_sav_mode = 'do_backups_with_extension'`.

#### `mc_editor_check_new_line`
Check POSIX new line.

#### `mc_editor_line_state`
See `mcedit(1)`.

#### `mc_editor_stop_format_char`
Compare `mcedit(1)`.

#### `mc_editor_wrdcompl_coll_ent_f`
Search autocomplete candidates in entire of file
or just from begin of file to cursor position (0).

#### `mc_editor_simple_statusbar`
Whether to use a simple statusbar or not.

#### `mc_editor_show_right_margin`
Whether to show a right margin
by inverting foreground and background colors beyond it.

#### `mc_editor_filesize_threshold`
Maximum filesize the editor is willing to edit.

#### `mc_mcview_eof`
Displayed string to mark EOF in `mcview(1)`.

#### `mc_mcview_remember_file_pos`
Probably does what it says.

#### `mc_max_dirt_limit`
See `mc(1)` for an explanation.

#### `mc_mouse_move_page_viewer`
See `mc(1)` for an explanation.

#### `mc_only_leading_plus_minus`
See `mc(1)` for an explanation.

#### `mc_show_output_starts_shell`
See `mc(1)` for an explanation.

#### `mc_use_file_to_guess_type`
See `mc(1)` for an explanation.

#### `mc_xtree_mode`
See `mc(1)` for an explanation.

#### `mc_fish_directory_timeout`
See `mc(1)` for an explanation.

#### `mc_mouse_close_dialog`
Undocumented.

#### `mc_mouse_repeat_rate`
Undocumented.

#### `mc_double_click_speed`
Undocumented.

#### `mc_ftpfs_retry_seconds`
Some limit.

#### `mc_ftpfs_use_unix_lst_option`
Whether to use UNIX list options.

#### `mc_ftpfs_first_cd_then_ls`
Whether to do `ls` after `cd`.

#### `mc_ignore_ftp_chattr_errors`
Whether to ignore `chattr(1)` errors.

#### `mc_keymap`
The name of the keymap to use.

#### `mc_alternate_plus_minus`
Whether to alternate plus and minus.

#### `mc_confirm_view_dir`
Whether it's necessary to confirm viewing a directory.

#### `mc_clear_before_exec`
Whether to `clear(1)` before executing the selected file.

#### `mc_wrap_mode`
Whether wrap mode is enabled.

#### `mc_fast_refresh`
Whether to make fast `refresh(3ncurses)`es.

#### `mc_num_history_items_recorded`
The maximum number of items the history records.

#### `mc_select_flags`
???

#### `mc_copymove_persistent_attr`
Whether to enforce `chattr(1)`/`attr(5)` attributes got copymoved, too.

#### `mc_horizontal_split`
Whether to horizontal split thus dividing the view into a tob and bottom panel.

#### `mc_kilobyte_si`
Whether to use
[SI-prefixes](https://secure.wikimedia.org/wikipedia/en/wiki/Metric_prefix)
kilo- and Mega- instead of
ki[bi-](https://secure.wikimedia.org/wikipedia/en/wiki/Binary_prefix) and Mebi-.

#### `mc_mix_all_files`
Disable showing all directories alphabetically first and then normal files.

#### `mc_show_backups`
Whether to list backup files with the known `mc_editor_backup_extension`.

#### `mc_show_dot_files`
Whether to list files whose filenames start with a `.` (dot).

#### `mc_fast_reload`
Do fast reload.

#### `mc_fast_reload_msg_shown`
Whether the message of using fast reload was shown.

#### `mc_mark_moves_down`
Whether marking a file also moves the cursor down.

#### `mc_reverse_files_only`
Reverse files only.

#### `mc_auto_save_setup_panels`
Enables automatically saving the panels setup.

#### `mc_navigate_with_arrows`
Allows navigation with arrows.

#### `mc_mouse_moves_pages`
Whether the rat can move pages, too.

#### `mc_mouse_moves_pages_viewer`
Whether the rate can move pages in the viewer, too.

#### `mc_panel_scroll_pages`
Scroll pages in panels.

#### `mc_torben_fj_mode`
Switches into `torben_fj_mode`.

#### `mc_layout_horizontal_split`
Whether to split into top and bottom parts.

#### `mc_layout_equal_split`
By default, panels have equal sizes.

#### `mc_layout_first_panel_size`
???

#### `mc_layout_horizontal_equal`
Do equally split between top and bottom panels.

#### `mc_layout_top_panel_size`
The height of the top panel, if not `mc_layout_horizontal_equal`.

#### `mc_layout_vertical_equal`
Do equally split between left and right panels.

#### `mc_layout_left_panel_size`
The width of the left panel, if not `mc_layout_vertical_equal`.

#### `mc_layout_output_lines`
On the Linux or FreeBSD console
you can specify how many lines are shown in the output window.
This option is available if Midnight Commander runs on native console only.

#### `mc_layout_menubar_visible`
If enabled,
the main menu of midnight commander
is always visible on the top row of screen above panels.
Enabled by default.

#### `mc_layout_command_prompt`
If enabled, command line is avalable.
Enabled by default.

#### `mc_layout_keybar_visible`
If enabled,
ten lables associated with `F1`–`F10` keys
are located at the bottom row of screen.
Enabled by default.

#### `mc_layout_message_visible`
If enabled, the one-line hints are visible below panels.
Enabled by default.

#### `mc_layout_xterm_title`
When run in a terminal emulator for X11,
midnight commander sets the terminal window title
to the current working directory and updates it when necessary.
If your terminal emulator is broken
and you see some incorrect output on startup and directory change,
turn off this option.
Enabled by default.

#### `mc_layout_free_space`
If enabled, free space and total space of current file system
is shown at the bottom frame of panel.
Enabled by default.

#### `mc_layout_show_mini_info`
???

#### `mc_layout_filetype_mode`
Colorcode filetype mode.

#### `mc_layout_permission_mode`
Colorcode permissions.

#### `mc_colors_color_terminals`
List of terminals `mc(1)` always forces color mode with.

#### `mc_panels_show_mini_info`
If enabled,
one line of status information about the currently selected item
is shown at the bottom of the panels.
Enabled by default.

#### `mc_panels_kilobyte_si`
If this option is enabled,
midnight commander will use SI units (powers of 1000)
when displaying any byte sizes.
The suffixes (k, m, …) are shown in lowercase.
If disabled (default),
midnight commander will use binary units (powers of 1024)
and the suffixes are shown in upper case (K, M, …).

#### `mc_panels_mix_all_files`
If this option is enabled,
all files and directories are shown mixed together.
If the option is disabled (default),
directories (and links to directories)
are shown at the beginning of the listing,
and other files below.

#### `mc_panels_show_backups`
If enabled, the midnight commander will show files ending with a tilde.
Otherwise, they won't be shown (like GNU's `ls(1)` option `-B`).
Enabled by default.

#### `mc_panels_show_dot_files`
If enabled,
the midnight commander will show all files
that start with a dot (like `ls -a`).
Disabled by default.

#### `mc_panels_fast_reload`
If this option is enabled,
the midnight commander will use a trick
to determine if the directory contents have changed.
The trick is to reload the directory
only if the inode of the directory has changed;
this means that reloads only happen when files are created or deleted.
If what changes is the inode for a file in the directory
(file size changes, mode or owner changes, etc.)
the display is not updated.
In these cases,
if you have the option on,
you have to rescan the directory manually (with `C-r`).
Disabled by default.

#### `mc_panels_fast_relod_msg_shwn`
???

#### `mc_panels_mark_moves_down`
If enabled,
the selection bar will move down when you mark a file (with `insert` key).
Enabled by default.

#### `mc_panels_reverse_files_only`
Allow revert selection of files only.
Enabled by default.
If enabled,
the reverse selection is applied to files only,
not to directories.
The selection of directories is untouched.
If off, the reverse selection is applied to files as well to directories:
all unselected items become selected, and vice versa.

#### `mc_panels_simple_swap`
If both panels contain file listings,
simple swap means
that panels exchange their screen positions:
the left panel becomes the right one, and vice versa.
If this option is unchecked,
file listing panels exchange their content
keeping listing format and sort options.
Unchecked by default.

#### `mc_panels_auto_sav_setup_pnls`
If this option is enabled,
when you exit the midnight commander
the current settings of panels are saved in the `~/.config/mc/panels.ini` file.
Disabled by default.

#### `mc_panels_navigate_w_arrows`
If this option is enabled,
you may use the arrows keys
to automatically chdir
if the current selection is a subdirectory
and the shell command line is empty.
By default, this setting is off.

#### `mc_panels_panel_scroll_pages`
If set (the default), panel will scroll by half the display
when the cursor reaches the end or the beginning of the panel,
otherwise it will just scroll a file at a time.

#### `mc_panels_mouse_move_pages`
Controls whenever scrolling with the mouse wheel
is done by pages or line by line on the panels.

#### `mc_panels_torben_fj_mode`
See `mc(1)`.

#### `mc_panels_filetype_mode`
If the file type highlighting is enabled,
file names are colored according to rules
described in `/etc/mc/filehighlight.ini` file.

#### `mc_panels_permission_mode`
You can specify
whether permissions and file types
should be highlighted with distinctive colors.
If the permission highlighting is enabled,
the parts of the perm and mode display fields
which apply to the user running midnight commander
are highlighted with the color defined by the selected keyword.

#### `mc_panels_quick_search_mode`
You can specify how the quick search mode should work:
`case_insensitive`ly, `case_sensitive`ly or
be matched to the the panel sort order (`use_panel_sort_mode`):
case sensitive or not.

#### `mc_misc_display_codepage`
Display code page.

#### `mc_misc_ftpfs_password`
FTP FS password.

#### `mc_misc_ftp_proxy_host`
Hostname of FTP proxy.

#### `mc_misc_timeformat_recent`
Compare `mc(1)`.

#### `mc_misc_timeformat_old`
Compare `mc(1)`.

#### `mc_misc_clipboard_store`
Compare `mc(1)`.

#### `mc_misc_clipboard_paste`
Compare `mc(1)`

#### `mc_misc_autodetect_codeset`
Compare `mc(1)`

#### `mc_misc_spell_language`
???

#### `mc_misc_source_codepage`
???

[//]: # ( #### `mc_misc_find_ignore_dirs` )
[//]: # ( ??? )


## Limitations

* This module's been used with stable Debian and Ubuntu releases.
* It can't uninstall `mc` or its changes.
* `/usr/share/mc/mc.ini` gets _completly_ copied to `~/.config/mc/ini`
  the user invokes `mc` the first time
  (if `~/.config/mc/ini` is absent, to be precise).
  Subsequent changes in `/usr/share/mc/mc.ini`
  won't be considered automatically.


## Development

Drop me a line (`patch(1)`)
`Kai Burghardt <wiz?KaiBurghardt.de>`
(GnuPG usage advised).
This module's primarily targeted for
[already packaged stable open source Puppet](https://docs.puppet.com/guides/puppetlabs_package_repositories.html)
 versions.

[//]: # ( vim: set ft=markdown tw=80 ts=4 noet bs=eol,start,indent:)
